package org.ovirt.engine.ui.common.widget.refresh;

import com.google.gwt.resources.client.CssResource;

public interface BaseRefreshPanelCss extends CssResource {

    String checkitem();

    String popupBottomCenter();

    String popupBottomLeft();

    String popupBottomLeftInner();

    String popupBottomRight();

    String popupBottomRightInner();

    String popupMiddleLeft();

    String popupMiddleRight();

    String popupTopCenter();

    String popupTopLeft();

    String popupTopLeftInner();

    String popupTopRight();

    String popupTopRightInner();

    String refreshButton();

    String refreshButtonDownHovering();

    String refreshIcon();

    String refreshMenuButton();

    String refreshMenuTitle();

    String refreshRateMenu();

    String refreshRateMenuPopup();

    String refreshRateOption();

    String refreshRateOptionSelected();

    String textItem();

}
